@@ -27,10 +27,17 @@ Basic context-free grammars manipulation for building uniform random generetors
 #__package__ = "FAdo"
 
 import re
-import string
-from types import StringType
+#import string
 from random import randint
-import common
+
+#import common
+
+from . import common
+
+# This is a workarround for python3
+# TODO: fix it
+StringType = type("a")
+
 
 class CFGrammar(object):
     """ Class for context-free grammars
@@ -63,7 +70,7 @@ class CFGrammar(object):
         self.Nullable = {}
         self.tr = {}
         self.ntr = {}
-        for i in xrange(len(self.Rules)):
+        for i in range(len(self.Rules)):
             if self.Rules[i][0] not in self.ntr:
                 self.ntr[self.Rules[i][0]] = {i}
             else:
@@ -74,11 +81,11 @@ class CFGrammar(object):
 
         :return: a string representing the grammar rules"""
         s = ""
-        for n in xrange(len(self.Rules)):
+        for n in range(len(self.Rules)):
             lhs = self.Rules[n][0]
             rhs = self.Rules[n][1]
             if type(rhs) is not StringType and len(rhs) > 1:
-                rhs = string.join(rhs)
+                rhs = str.join(rhs)
             s += "{0:d} | {1:s} -> {2:s} \n".format(n, lhs, rhs)
         return "Grammar Rules:\n\n%s" % s
 
@@ -102,7 +109,7 @@ class CFGrammar(object):
     def terminalrules(self):
         self.tr = {}
         for a in self.Terminals:
-            for i in xrange(len(self.Rules)):
+            for i in range(len(self.Rules)):
                 if self.Rules[i][1] == a:
                     if a not in self.tr:
                         self.tr[a] = {i}
@@ -111,7 +118,7 @@ class CFGrammar(object):
 
     def nonterminalrules(self):
         self.ntr = {}
-        for i in xrange(len(self.Rules)):
+        for i in range(len(self.Rules)):
             if self.Rules[i][0] not in self.ntr:
                 self.ntr[self.Rules[i][0]] = {i}
             else:
@@ -206,17 +213,17 @@ class CNF(CFGrammar):
         self.nttr = {}
         # terminal a is replaced by A@_a in all rules > 2
         for a in self.Terminals:
-            for i in xrange(len(self.Rules)):
+            for i in range(len(self.Rules)):
                 if type(self.Rules[i][1]) is not StringType and len(self.Rules[i][1]) >= 2 and a in self.Rules[i][1]:
                     if a not in self.nttr:
                         self.nttr[a] = self.get_ntr_tr(a)
                     rr = list(self.Rules[i][1])
-                    for k in xrange(len(rr)):
+                    for k in range(len(rr)):
                         if rr[k] == a:
                             rr[k] = self.nttr[a]
                     self.Rules[i] = (self.Rules[i][0], tuple(rr))
         n = len(self.Rules)
-        for i in xrange(n):
+        for i in range(n):
             if type(self.Rules[i][1]) is not StringType and len(self.Rules[i][1]) > 2:
                 self.iter_rule(self.Rules[i][0], self.Rules[i][1], i)
 
@@ -273,7 +280,7 @@ class cfgGenerator(object):
                             break
             uk = randint(1, self.density_r[ic][n])
             rk = 1
-            for k in xrange(1, n):
+            for k in range(1, n):
                 if (k in self.density[g.Rules[ic][1][0]] and self.density[g.Rules[ic][1][0]][k] > 0 and
                             n - k in self.density[g.Rules[ic][1][1]] and self.density[g.Rules[ic][1][1]][
                         n - k] > 0):
@@ -298,7 +305,7 @@ class cfgGenerator(object):
         for t in g.tr:
             for i in g.tr[t]:
                 self.density[g.Rules[i][0]][1] += 1
-        for l in xrange(2, n + 1):
+        for l in range(2, n + 1):
             for nt in g.ntr:
                 r = 0
                 for i in g.ntr[nt]:
@@ -307,7 +314,7 @@ class cfgGenerator(object):
                             self.density_r[i] = {}
                         self.density_r[i][l] = sum(
                             [self.density[g.Rules[i][1][0]][k] * self.density[g.Rules[i][1][1]][l - k] for k in
-                             xrange(1, l) if
+                             range(1, l) if
                              k in self.density[g.Rules[i][1][0]] and l - k in self.density[g.Rules[i][1][1]]])
                         r += self.density_r[i][l]
                 if r:
@@ -385,16 +392,16 @@ def gRules(rules_list, rulesym="->", rhssep=None, rulesep='|'):
             if m.end() == len(rule):
                 raise common.CFGgrammarError(rule)
             else:
-                rest = string.strip(rule[m.end():])
+                rest = str.strip(rule[m.end():])
                 if rest == "[]":
                     rhs = []
                 else:
-                    multi = string.split(rest, rulesep)
+                    multi = str.split(rest, rulesep)
                     rhs = []
                     for i in multi:
-                        l = string.split(i, rhssep)
+                        l = str.split(i, rhssep)
                         if len(l) > 1:
-                            l = tuple(string.split(i, rhssep))
+                            l = tuple(str.split(i, rhssep))
                         else:
                             l = l[0]
                         gr.append((lhs, l))
@@ -411,11 +418,11 @@ def smallAlphabet(k, sigma_base="a"):
     if k >= 52:
         raise common.CFGterminalError(k)
     lim = min(26, k)
-    for i in xrange(lim):
+    for i in range(lim):
         Sigma.append(chr(ord(sigma_base) + i))
     if k >= 26:
         sigma_base = 'A'
-        for i in xrange(k - lim):
+        for i in range(k - lim):
             Sigma.append(chr(ord(sigma_base) + i))
     return Sigma
 
@@ -482,5 +489,10 @@ reGrammar = {'g_regular_base': ["Tr -> Tr + Tc | Tc", "Tc -> Tc Ts |  Ts",
                       "Tf -> {0:s} Tf Te | Te".format(common.Conj),
                       "Te -> {0:s} Ts | Ts ".format(common.Not),
                        "Ts -> . Ts Tu | . {0:s} Tu".format(common.SigmaS),
-                      "Tu ->  . Ti {0:s}".format(common.SigmaS)]
+                          "Tu ->  . Ti {0:s}".format(common.SigmaS)],
+             '2D_g_rpn': [ "Tx -> / R R | +  Tx  Tx  | . Tx Tx  | * Tx",
+                            "R -> {0:s} | Ti | +  R  R  | . R R  | * R".format(common.Epsilon)],
+               '2D_gR_rpn': [ "Tx -> / R R ",
+                            "R -> {0:s} | Ti | +  R  R  | . R R  | * R".format(common.Epsilon)],
+                           
             }
