@@ -24,12 +24,12 @@
    675 Mass Ave, Cambridge, MA 02139, USA."""
 import copy
 
-import reex
-from common import *
-import fio
-from transducers import SFT, infixTransducer, prefixTransducer, suffixTransducer, outfixTransducer, hypercodeTransducer
-import fa
-import fl
+from . import reex
+from . common import *
+from . import fio
+from . transducers import SFT, infixTransducer, prefixTransducer, suffixTransducer, outfixTransducer, hypercodeTransducer
+from . import fa
+from . import fl
 
 
 def fixedHierSubset(x, y):
@@ -310,11 +310,11 @@ class IPTProp(CodeProperty):
         :param int n: number of words to try
         :return: a string or None
         :rtype: str"""
-        #original
-        #if hasattr(self, "AutDetect"):
-        # b = self.AutDetect.runOnNFA(aut).trim()
-        #else:
-        #  b = (self.Aut | self.Aut.inverse()).runOnNFA(aut).trim()
+        # original
+        # if hasattr(self, "AutDetect"):
+        #   b = self.AutDetect.runOnNFA(aut).trim()
+        # else:
+        #   b = (self.Aut | self.Aut.inverse()).runOnNFA(aut).trim()
         b = (self.Aut | self.Aut.inverse()).inIntersectionSlow(aut).toOutNFA().trim()
         s = len(aut.Sigma)
         # can use ell = len(aut.witness()),  if ell was not given
@@ -377,23 +377,23 @@ class IPTProp(CodeProperty):
 
         a = fl.sigmaInitialSegment(set([str(i) for i in range(s)]), ell, True)
         g = fl.RndWGen(a)
-        w = g.next()
+        w = next(g)
         if ov_free and end is not None:
             l_end = len(end)
             end = list(end)
             we = w[-l_end:]
             while we != end or not overlapFreeP(w):
-                w = g.next()
+                w = next(g)
                 we = w[-l_end:]
         if ov_free and end is None:
             while not overlapFreeP(w):
-                w = g.next()
+                w = next(g)
         if not ov_free and end is not None:
             l_end= len(end)
             end = list(end)
             we = w[-l_end:]
             while we != end:
-                w = g.next()
+                w = next(g)
                 we = w[-l_end:]
         W = [w]
         aut = fl.FL(W).MADFA()#toNFA()
@@ -403,7 +403,7 @@ class IPTProp(CodeProperty):
             b = self.AutDetect.runOnNFA(aut).trim()
             tr = 1
             while tr <= n:
-                w = g.next()
+                w = next(g)
                 if ov_free:
                     if not overlapFreeP(w):
                         continue
@@ -1033,13 +1033,13 @@ def exponentialDensityP(aut):
 
     :param NFA aut: the representation of the language
     :rtype: bool"""
-    import fractions
+    import math
     from collections import deque
 
     def _listgcd(ll):
         if len(ll) == 1:
             return ll[0]
-        return fractions.gcd(ll[0], _listgcd(ll[1:]))
+        return math.gcd(ll[0], _listgcd(ll[1:]))
 
     components = aut.trim().stronglyConnectedComponents()
     for c in components:
@@ -1098,7 +1098,7 @@ def long2base(n, q):
     lt = []
     while n > 0:
         lt.append(n % q)
-        n /= q
+        n //= q
     return lt
 
 
