@@ -25,14 +25,14 @@ Combined operations
    with this program; if not, write to the Free Software Foundation, Inc.,
    675 Mass Ave, Cambridge, MA 02139, USA."""
 
-from fa import *
-from common import *
+from . fa import *
+from . common import *
 
 
 def starConcat(fa1, fa2, strict=False):
     """ Star of concatenation of two languages: (L1.L2)*
 
-    :param DFA f a1: first automaton
+    :param DFA fa1: first automaton
     :param DFA fa2: second automaton
     :param bool strict: should the alphabets be necessary equal?
     :rtype: DFA
