@@ -35,16 +35,25 @@ import subprocess
 import warnings
 import re
 from itertools import chain, combinations
+from functools import reduce
+
 try:
-    from IPython.display import display, SVG, get_ipython
+    import __pypy__
+    PyPy = True
+except ImportError:
+    PyPy = False
+
+try:
+    from IPython.display import display, SVG  # , get_ipython
 except:
     pass
 
-FAdoVersion = "1.3.5.1"
+FAdoVersion = "2.0"
+__version__ = FAdoVersion
+
 
 def run_from_ipython_notebook():
     try:
-        __IPYTHON__
         cfg = get_ipython().config
         if 'IPKernelApp' in cfg:
             return True
@@ -53,6 +62,7 @@ def run_from_ipython_notebook():
     except NameError:
         return False
 
+
 MAXLBL = 10
 
 
@@ -80,6 +90,10 @@ class DFAerror(fnhException):
     pass
 
 
+class NFAerror(fnhException):
+    pass
+
+
 class TFASignal(fnhException):
     pass
 
@@ -104,6 +118,9 @@ class FAdoGeneralError(FAdoError):
         return "FAdo: " + self.msg
 
 
+class TypeError(FAdoError):
+    pass
+
 class VersoError(FAdoGeneralError):
     pass
 
@@ -260,11 +277,28 @@ class DFAmarkedError(DFAerror):
         return "Symbol not marked %s" % str(self.sym)
 
 
+class NFAEmpty(NFAerror):
+    def __str__(self):
+        return "Nfa is empty"
+
+
 class TRError(FAException):
     def __str__(self):
         return "Transducer Error"
 
 
+class SSError(FAdoError):
+    pass
+
+
+class SSMissAlphabet(SSError):
+    def __str__(self):
+        return "Missing alphabet"
+
+
+class SSBadTransition(SSError):
+    def __str__(self):
+        return "Bad empty transition"
 
 
 class regexpInvalid(DFAerror):
@@ -283,13 +317,15 @@ class regexpInvalidSymbols(DFAerror):
     def __str__(self):
         return "%s" % self.message
 
+
 class regexpInvalidMethod(FAdoGeneralError):
     def __init__(self):
-        self.message = 'Method not Immplemented for %s' %str(type(self))
+        self.message = 'Method not Immplemented for %s' % str(type(self))
 
     def __str__(self):
         return "%s" % self.message
 
+
 class PEGError(FAdoGeneralError):
     pass
 
@@ -329,7 +365,7 @@ class GraphError(fnhException):
         return "%s" % self.message
 
 
-class TestsError(DFAerror):
+class TstError(DFAerror):
     def __init__(self, message):
         self.message = message
 
@@ -349,6 +385,7 @@ class NotSP(DFAerror):
     def __str__(self):
         return "DFA is not Serial-Paralel."
 
+
 """
 :var EmptySet: default representation for empty set
 :var Epsilon: default representation for epsilon
@@ -368,6 +405,7 @@ SigmaS = "@sigmaS"
 
 DEBUG = False
 
+TYPE_EPSILON = "epsilon"
 TYPE_DISJ = "disj"
 TYPE_CONC = "concat"
 TYPE_STAR = "star"
@@ -375,8 +413,18 @@ TYPE_SYMB = "sym"
 TYPE_EWRD = "ewrd"
 TYPE_ESET = "eset"
 TYPE_CONJ = "conj"
+TYPE_OPTION = "option"
 TYPE_SHUFFLE = "shuffle"
 
+ID_EPSILON = 0
+ID_DISJ = 1
+ID_CONC = 2
+ID_STAR = 3
+ID_SYMB = 4
+ID_CONJ = 5
+ID_OPTION = 6
+ID_SHUFFLE = 7
+ID_EMPTYSET = 8
 
 def if_else(a, b, c):
     if a:
@@ -385,8 +433,7 @@ def if_else(a, b, c):
 
 
 def debug(string, level=0):
-    print "%s%s" % ("".join(["\t" for _ in xrange(level)]), string)
-
+    print("%s%s" % ("".join(["\t" for _ in range(level)]), string))
 
 
 class SPLabel(object):
@@ -467,6 +514,7 @@ class memoized(object):
     """Decorator that caches a function's return value each time it is called.
 
     If called later with the same arguments, the cached value is returned, and not re-evaluated."""
+
     def __init__(self, func):
         self.func = func
         self.cache = {}
@@ -547,6 +595,7 @@ def dememoize(cls, method_name):
         del cls.memoized_instances
     return True
 
+
 try:
     from itertools import product as cartesianProduct
 except ImportError:
@@ -561,6 +610,7 @@ def uSet(s):
     :return: the first element of s"""
     return list(s)[0]
 
+
 def lSet(s):
     """returns the last element of a set
 
@@ -570,6 +620,7 @@ def lSet(s):
     .. versionadded:: 1.3.3"""
     return list(s)[-1]
 
+
 def tmpFileName():
     i = os.getpid()
     r = random.randint(0, 1000000)
@@ -594,7 +645,7 @@ def binomial(n, k):
     :param int n: n
     :param int k: k
     :rtype: int"""
-    return reduce(lambda acc, m: acc * (n - k + m) / m, range(1, k + 1), 1)
+    return reduce(lambda acc, m: acc * (n - k + m) / m, list(range(1, k + 1)), 1)
 
 
 def delFromList(l, l1):
@@ -618,6 +669,14 @@ def suffixes(word):
     .. versionadded: 0.9.8"""
     return [word[i:] for i in range(1, len(word))]
 
+
+def prefixP(word, prefix):
+    if prefix == Epsilon:
+        return True
+    else:
+        return word[:len(prefix)] == prefix
+
+
 def overlapFreeP(word):
     """
     Returns True if word is overlap free, i.e, no  proper  and nonempty
@@ -627,16 +686,17 @@ def overlapFreeP(word):
     :rtype: Boolean
     """
     l = len(word)
-    for i in range(l-1):
+    for i in range(l - 1):
         foo = True
-        for j in range(i+1):
-            if word[j] != word[-(i+1)+j]:
+        for j in range(i + 1):
+            if word[j] != word[-(i + 1) + j]:
                 foo = False
                 break
         if foo:
             return False
     return True
 
+
 def graphvizTranslate(s, strict=False, maxLblSz=6):
     """Translate epsilons for graphviz
 
@@ -644,28 +704,31 @@ def graphvizTranslate(s, strict=False, maxLblSz=6):
     :arg maxLblSz: max size of labels before getting removed
     :param bool strict: use limitations of label sizes
     :rtype: str"""
-    return re.sub(Epsilon,"&epsilon;",s)
+    return re.sub(Epsilon, "&epsilon;", s)
 
 
 def powerset_generator(i):
-    for subset in chain.from_iterable(combinations(i, r) for r in range(len(i) + 1)):
+    for subset in chain.from_iterable(combinations(i, r) for r in list(range(len(i) + 1))):
         yield set(subset)
 
-
-def deprecated(func):
-    """This is a decorator which can be used to mark functions as deprecated. It will result in a warning being emmitted
-    when the function is used."""
-    def newFunc(*args, **kwargs):
-        warnings.warn("Call to deprecated function %s." % func.__name__, category=DeprecationWarning)
-        return func(*args, **kwargs)
-    newFunc.__name__ = func.__name__
-    newFunc.__doc__ = func.__doc__
-    newFunc.__dict__.update(func.__dict__)
-    return newFunc
+#
+# def deprecated(func):
+#     """This is a decorator which can be used to mark functions as deprecated. It will result in a warning being emmitted
+#     when the function is used."""
+#
+#     def newFunc(*args, **kwargs):
+#         warnings.warn("Call to deprecated function %s." % func.__name__, category=DeprecationWarning)
+#         return func(*args, **kwargs)
+#
+#     newFunc.__name__ = func.__name__
+#     newFunc.__doc__ = func.__doc__
+#     newFunc.__dict__.update(func.__dict__)
+#     return newFunc
 
 
 class Drawable(object):
     """Any FAdo object that is drawable"""
+
     def display(self, fileName=None, size="30,20", strict=False, maxLblSz=6):
         """ Display automata using dot
 
@@ -699,7 +762,7 @@ class Drawable(object):
             callstr = "dot -Tpdf %s -o %s" % (fnameGV, filenameOut)
         result = subprocess.call(callstr, shell=True)
         if result:
-            print "Need graphviz to visualize objects"
+            print("Need graphviz to visualize objects")
             return
         if run_from_ipython_notebook():
             display(SVG(filename=filenameOut))
@@ -731,12 +794,31 @@ class Drawable(object):
         callstr = "dot -Tpng %s -o %s" % (fnameGV, fnamePNG)
         result = subprocess.call(callstr, shell=True)
         if result:
-            print "Need graphviz to visualize objects"
+            print("Need graphviz to visualize objects")
             return
         return fnamePNG
+    
+    def dotLabel(self,lbl):
+        """ Label string
 
+        """
+        if type(lbl) == tuple:
+            return "({0:s} , {1:s})".format(self.dotLabel(lbl[0]), self.dotLabel(lbl[1]))
+        elif type(lbl) == set or type(lbl) == list:
+            lbl = list(lbl)
+            if  len(lbl) == 0:
+                return str(lbl)
+            if len(lbl) == 1:
+                return "{0:s}".format(self.dotLabel(lbl[0]))
+            stl = "{0:s}".format(self.dotLabel(lbl[0]))
+            for s in lbl[1:]:
+                stl += ", {0:s}".format(self.dotLabel(s))
+            return stl
+        else:
+            return str(lbl)
+        
     @abstractmethod
-    def dotFormat(self, size):
+    def dotFormat(self, size="20,20", fileName=None, direction="LR", strict=False, maxLblSz=6, sep="\n"):
         """Some dot representation
 
         :param str size: size parameter for dotviz
@@ -747,7 +829,8 @@ class Drawable(object):
 class Word(object):
     """Class to implement generic words as iterables with pretty-print
 
-    Basically a unified way to deal with words with caracters of of sizes different of one with no much fuss"""
+    Basically a unified way to deal with words with caracters of sizes different of one with no much fuss"""
+
     def __init__(self, data=None, it=None):
         if it is not None:
             for c in it:
@@ -793,31 +876,59 @@ class Word(object):
             return True
         else:
             return self.data == other.data
-
-    def __cmp__(self, other):
+    def __gt__(self, other):
         a, b = len(self.data), len(other.data)
         if a == b:
-            return cmp(self.data, other.data)
-            # if self.data < other.data:
-            #     return -1
-            # elif self.data > other.data:
-            #     return 1
-            # else:
-            #     return 0
+            return (self.data > other.data)
         elif a < b:
-            if self.data < other.data[:a+1]:
-                return -1
-            elif self.data > other.data[:a+1]:
-                return 1
+            if self.data > other.data[:a + 1]:
+                return True
             else:
-                return -1
+                return False
         else:
-            if self.data[:b+1] < other.data:
-                return -1
-            elif self.data[:b+1] > other.data:
-                return 1
+            if self.data[:b + 1] < other.data:
+                return False
             else:
-                return 1
+                return True
+
+    def __lt__(self, other):
+        if self == other:
+            return False
+        return not self > other
+
+    def __ne__(self, other):
+        return not self == other
+
+    def __le__(self, other):
+        return self == other or self < other
+
+    def __ge__(self, other):
+        return self == other or self > other
+
+    # def __cmp__(self, other):
+    #     a, b = len(self.data), len(other.data)
+    #     if a == b:
+    #         return (self.data > other.data) - (self.data < other.data)
+    #         # if self.data < other.data:
+    #         #     return -1
+    #         # elif self.data > other.data:
+    #         #     return 1
+    #         # else:
+    #         #     return 0
+    #     elif a < b:
+    #         if self.data < other.data[:a + 1]:
+    #             return -1
+    #         elif self.data > other.data[:a + 1]:
+    #             return 1
+    #         else:
+    #             return -1
+    #     else:
+    #         if self.data[:b + 1] < other.data:
+    #             return -1
+    #         elif self.data[:b + 1] > other.data:
+    #             return 1
+    #         else:
+    #             return 1
 
     # def __add__(self, other):
     #     new = deepcopy(self)
@@ -851,6 +962,7 @@ class Word(object):
 
 class AllWords:
     """ Iterator thar generates all words of an alphabet in militar order """
+
     def __init__(self, alphabet):
         """
         :type alphabet: list|set"""
@@ -891,7 +1003,7 @@ class AllWords:
         else:
             return w[:k] + [0 for _ in range(l - k)]
 
-    def next(self):
+    def __next__(self):
         self.last = self._next(self.last, self.wl, self.wl - 1)
         return Word(self._translate())
 
@@ -925,22 +1037,25 @@ def homogeneousP(l):
 
 class twDict(object):
     """A class for dictionaries 'both ways' """
-    def __init__(self, fw={}):
-        self.fw, self.bw  = dict(), dict()
+
+    def __init__(self, fw=None):
+        if fw is None:
+            fw = {}
+        self.fw, self.bw, self.mult = dict(), dict(), dict()
         for k in fw:
             foo = fw[k]
             self.fw[k] = foo
             self.bw[foo] = self.bw.get(foo, set()).add(k)
-            self.mult[foo] = self.mult.get(foo,0) + 1
-
+            self.mult[foo] = self.mult.get(foo, 0) + 1
 
     def set(self, k, val):
+        foo = None
         if k in self.fw:
             foo = self.fw[k]
-        self.fw[k] =val
+        self.fw[k] = val
         self.bw[val] = self.bw.get(val, set()).add(k)
         if len(self.bw[foo]) == 1:
-            del(self.bw[foo])
+            del (self.bw[foo])
         else:
             self.bw[foo].discard(k)
 
@@ -949,3 +1064,40 @@ class twDict(object):
 
     def getB(self, v):
         return self.bw[v]
+
+
+def getOneFromSet(s):
+    return next(iter(s))
+
+
+def sConcat(x, y):
+    """ Concat words
+
+    :param x: first word
+    :param y: second word
+    :return: concatenation word"""
+    if x == Epsilon:
+        return y
+    elif y == Epsilon:
+        return x
+    else:
+        return x + y
+#
+# def cmp_to_key(mycmp):
+#     'Convert a cmp= function into a key= function'
+#     class K:
+#         def __init__(self, obj, *args):
+#             self.obj = obj
+#         def __lt__(self, other):
+#             return mycmp(self.obj, other.obj) < 0
+#         def __gt__(self, other):
+#             return mycmp(self.obj, other.obj) > 0
+#         def __eq__(self, other):
+#             return mycmp(self.obj, other.obj) == 0
+#         def __le__(self, other):
+#             return mycmp(self.obj, other.obj) <= 0
+#         def __ge__(self, other):
+#             return mycmp(self.obj, other.obj) >= 0
+#         def __ne__(self, other):
+#             return mycmp(self.obj, other.obj) != 0
+#     return K
