@@ -25,9 +25,9 @@ Finite languages manipulation
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA."""
 
-import fa
+from . import fa
 from copy import copy
-from common import *
+from . common import *
 import random
 
 
@@ -198,6 +198,13 @@ class FL(object):
         .. seealso:: Incremental Construction of Minimal Acyclic Finite-State Automata, J.Daciuk, S.Mihov, B.Watson and R.E.Watson
 
         :rtype: ADFA"""
+        if self.Words == FL([Epsilon]).Words:
+            aut = ADFA()
+            i = aut.addState()
+            aut.setInitial(i)
+            aut.addFinal(i)
+            aut.setSigma(self.Sigma)
+            return aut
         aut = ADFA()
         i = aut.addState()
         aut.setInitial(i)
@@ -316,13 +323,13 @@ class AFA(object):
         self.Final = set()
 
     @abstractmethod
-    def addState(self):
+    def addState(self, _):
         """
         :rtype: int"""
         pass
 
     @abstractmethod
-    def finalP(self):
+    def finalP(self, _):
         pass
 
     def setDeadState(self, sti):
@@ -358,7 +365,7 @@ class AFA(object):
         def _dealS(st1):
             if st1 not in torder:
                 torder.append(st1)
-                if st1 in self.delta.keys():
+                if st1 in list(self.delta.keys()):
                     for k in self.delta[st1]:
                         for dest in forceIterable(self.delta[st1][k]):
                             if dest not in torder and dest != self.Dead:
@@ -419,10 +426,10 @@ class AFA(object):
         :rtype: tuple"""
         (deltaC, rdelta) = self._getRdelta()
         rank, deltai = {}, {}
-        for s in xrange(len(self.States)):
+        for s in range(len(self.States)):
             deltai.setdefault(deltaC[s], set([])).add(s)
         i = -1
-        notDone = copy(range(len(self.States)))
+        notDone = copy(list(range(len(self.States))))
         deltaC[self.Dead] = 0
         deltai[1].remove(self.Dead)
         deltai[0] = {self.Dead}
@@ -505,7 +512,7 @@ class ADFA(fa.DFA, AFA):
                 self.Dead = foo
         for st in range(len(self.States)):
             for k in self.Sigma:
-                if k not in self.delta.get(st, {}).keys():
+                if k not in list(self.delta.get(st, {}).keys()):
                     self.addTransition(st, k, self.Dead)
         self.Minimal = False
         return self
@@ -559,15 +566,7 @@ class ADFA(fa.DFA, AFA):
         """ Conversion to DFCA
 
         :rtype: DFA"""
-        new = fa.DFA()
-        new.States = deepcopy(self.States)
-        new.Sigma = deepcopy(self.Sigma)
-        new.Initial = self.Initial
-        new.Final = copy(self.Final)
-        for s in self.delta:
-            for c in self.delta[s]:
-                new.addTransition(s, c, self.delta[s][c])
-        return new
+        return self.forceToDFA()
 
     def wordGenerator(self):
         """Creates a random word generator
@@ -624,7 +623,7 @@ class ADFA(fa.DFA, AFA):
         rank, rdelta = new.evalRank()
         toBeDeleted = []
         maxr = len(rank) - 2
-        for r in xrange(maxr + 1):
+        for r in range(maxr + 1):
             ls = _getListDelta(rank[r])
             (d0, s0) = ls[0]
             j = 1
@@ -646,9 +645,7 @@ class ADFA(fa.DFA, AFA):
 
         :rtype: ADFA"""
         new = self.dup()
-        rank, rdelta = new.evalRank()
-
-
+        new.evalRank()
 
     def statePairEquiv(self, s1, s2):
         """Tests if two states of a ADFA are equivalent
@@ -740,32 +737,33 @@ class ADFA(fa.DFA, AFA):
                 for c in self.delta.get(s, {}):
                     for s1 in self.delta[s][c]:
                         inCount[s1] = inCount.get(s1, 0) + 1
-            q = self.Initial
-            visited = [q]
-            for i, sym in enumerate(wrd):
-                if sym not in self.delta.get(s, {}) or inCount[self.delta[q][sym]] > 1:
-                    return q, i
-                q = self.delta[q][sym]
-                visited.append(q)
+            q1 = self.Initial
+            visited1 = [q1]
+            for ii, sym in enumerate(wrd):
+                if sym not in self.delta.get(q1, {}) or inCount[self.delta[q1][sym]] > 1:
+                    # here there was a reference to s self.delta.get(s, {}) that must be wrong!
+                    return q1, ii
+                q1 = self.delta[q1][sym]
+                visited1.append(q1)
 
         def _cloneConfluence(st, wrd, ind):
-            q = st
-            for i, sym in enumerate(wrd[ind:]):
-                if sym not in self.delta.get(q, {}):
-                    return q, i + ind
-                qn = self.delta[q][sym]
+            q1 = st
+            for ii, sym in enumerate(wrd[ind:]):
+                if sym not in self.delta.get(q1, {}):
+                    return q1, ii + ind
+                qn = self.delta[q1][sym]
                 sn = self.addState()
                 for c1 in self.delta.get(qn, {}):
                     self.delta.setdefault(sn, {})[c1] = self.delta[qn][c1]
                     if self.finalP(qn):
                         self.addFinal(sn)
-                self.addTransition(q, sym, sn)
-                q = sn
-                visited.append(q)
+                self.addTransition(q1, sym, sn)
+                q1 = sn
+                visited.append(q1)
 
         def _replOrReg(st, wrd):
             if len(w) != 0:
-                self.addTransition(q,wrd[0],_replOrReg(self.delta[st][wrd[0]], wrd[1:]))
+                self.addTransition(q, wrd[0], _replOrReg(self.delta[st][wrd[0]], wrd[1:]))
             else:
                 for c in register:
                     if self.statePairEquiv(c, q):
@@ -792,8 +790,6 @@ class ADFA(fa.DFA, AFA):
         if j < len(w):
             self.delta[f][w[j]] = _replOrReg(self.delta[f][w[j]], w[j+1:])
 
-
-
     def dissMin(self, witnesses=None):
         """Evaluates the minimal dissimilarity language
         :param dict witnesses: optional witness dictionay
@@ -936,7 +932,7 @@ class ADFA(fa.DFA, AFA):
         if not self.completeP():
             new.complete()
         rank = new.directRank()
-        irank = dict((v, [k for (k, xx) in filter(lambda (key, value): value == v, rank.items())])
+        irank = dict((v, [k for (k, xx) in [key_value for key_value in list(rank.items()) if key_value[1] == v]])
                      for v in set(rank.values()))
         l = rank[new.Initial]
         lvl = new.level()
@@ -984,7 +980,7 @@ class ADFA(fa.DFA, AFA):
         new = ANFA()
         new.setSigma(copy(self.Sigma))
         new.States = copy(self.States)
-        for s in xrange(len(self.States)):
+        for s in range(len(self.States)):
             for k in self.delta.get(s, {}):
                 new.addTransition(s, k, self.delta[s][k])
         new.addInitial(self.Initial)
@@ -1040,7 +1036,7 @@ class RndWGen(object):
             else:
                 r -= j
 
-    def next(self):
+    def __next__(self):
         """Next word
 
         :return: a new random word"""
@@ -1288,7 +1284,6 @@ def genRndTrieUnbalanced(maxL, Sigma, ratio, safe=True):
             return False
         else:
             return True
-
     if safe:
         ensurance = _genEnsurance(maxL, Sigma)
     else:
@@ -1318,7 +1313,7 @@ def genRandomTrie(maxL, Sigma, safe=True):
     def _genEnsurance(m, alphabet):
         l = len(alphabet)
         sl = list(alphabet)
-        return [sl[random.randint(0, l - 1)] for _ in xrange(m)]
+        return [sl[random.randint(0, l - 1)] for _ in range(m)]
 
     # noinspection PyUnboundLocalVariable
     def _descend(s1, ens, safe1, m):
@@ -1375,11 +1370,11 @@ def genRndTriePrefix(maxL, Sigma, ClosedP=False, safe=True):
     def _genEnsurance(m, alphabet):
         l = len(alphabet)
         sl = list(alphabet)
-        return [sl[random.randint(0, l - 1)] for _ in xrange(m)]
+        return [sl[random.randint(0, l - 1)] for _ in range(m)]
 
     def _descend(s1, ens, saf, m):
         sons = ClosedP
-        if m is 0:
+        if m == 0:
             final = random.randint(0, 1)
             if saf or final == 1:
                 trie.addFinal(s1)
@@ -1474,4 +1469,3 @@ def stringToADFA(s):
             new.addFinal(st)
     new.setInitial(len(s) - 1)
     return new
-
