@@ -26,12 +26,14 @@ Transducer manipulation.
    675 Mass Ave, Cambridge, MA 02139, USA."""
 
 import copy
-from exceptions import *
+import functools
+#from exceptions import *
 
-import fa
-import common
-from common import *
-import fl
+from . import fa
+from . import common
+from . common import *
+from . import fl
+from . common import Epsilon
 
 
 class ZERO(Exception):
@@ -39,7 +41,8 @@ class ZERO(Exception):
     pass
 
 
-def _concat((a, b)):
+def _concat(xxx_todo_changeme2):
+    (a, b) = xxx_todo_changeme2
     if a == Epsilon:
         return b
     elif b == Epsilon:
@@ -67,11 +70,11 @@ def isLimitExceed(NFA0Delta, NFA1Delta):
     :param dict NFA1Delta: NFA1's transition Delta
     :rtype: bool"""
     N = 0
-    for s in NFA0Delta.keys():
+    for s in list(NFA0Delta.keys()):
         for s1 in NFA0Delta[s]:
             N += len(NFA0Delta[s][s1])
     M = 0
-    for s in NFA1Delta.keys():
+    for s in list(NFA1Delta.keys()):
         for s1 in NFA1Delta[s]:
             M += len(NFA1Delta[s][s1])
     if N * N * M > 1000000:
@@ -149,10 +152,10 @@ class GFT(Transducer):
         :param str wo: word outputed"""
         if wi != Epsilon:
             for sym in wi:
-                self.Sigma.add(sym)
+                self.addSigma(sym)
         if wo != Epsilon:
             for sym in wo:
-                self.Output.add(sym)
+                self.addOutput(sym)
         if stsrc not in self.delta:
             self.delta[stsrc] = {wi: {(wo, sti2)}}
         elif wi not in self.delta[stsrc]:
@@ -194,13 +197,13 @@ class GFT(Transducer):
                                 cst = mst
                             new.addTransition(cst, wi[-1:], Epsilon, lst)
                         else:
-                            z = zip(wi, wo)
+                            z = list(zip(wi, wo))
                             n = len(wi)
                             m = len(wo)
                             if n > m:
-                                z += zip(wi[m:], [Epsilon] * (n - m))
+                                z += list(zip(wi[m:], [Epsilon] * (n - m)))
                             elif m > n:
-                                z += zip([Epsilon] * (m - n), wo[n:])
+                                z += list(zip([Epsilon] * (m - n), wo[n:]))
                             n = len(z)
                             for (symi, symo) in z[:n - 1]:
                                 mst = new.addState()
@@ -235,7 +238,7 @@ class GFT(Transducer):
             for c in self.delta[s]:
                 for (oc, s1) in self.delta[s][c]:
                     trList.append((s, c, oc, s1))
-        trList.sort(_comp)
+        trList.sort(key=functools.cmp_to_key(_comp))
         return trList
 
     def codeOfTransducer(self):
@@ -255,6 +258,16 @@ class GFT(Transducer):
         return 'GFT', _codeOfSet(self.Sigma) + _codeOfSet(self.Output) + _codeOfSet(self.Initial) + \
                _codeOfSet(self.Final) + str(self.listOfTransitions())
 
+    def addOutput(self, sym):
+        """Add a new symbol to the output alphabet
+
+        There is no problem with duplicate symbols because Output is a Set. No symbol Epsilon can be added
+
+        :param str sym: symbol or regular expression to be added"""
+        if sym == Epsilon:
+            raise common.DFAepsilonRedefinition()
+        self.Output.add(sym)
+
 
 class SFT(GFT):
     """Standard Form Tranducer
@@ -289,7 +302,7 @@ class SFT(GFT):
                     new.addTransition(i1, syi, syo, i2)
         for si in self.Initial:
             try:
-                i1 = new.stateIndex(self.States[si])
+                i1 = new.stateIndex(self.States[si],True)
             except common.DFAstateUnknown:
                 continue
             new.addInitial(i1)
@@ -317,8 +330,8 @@ class SFT(GFT):
             raise DFAstateUnknown(sti)
         if sti in self.delta:
             del self.delta[sti]
-        for j in self.delta.keys():
-            for sym in self.delta[j].keys():
+        for j in list(self.delta.keys()):
+            for sym in list(self.delta[j].keys()):
                 self._deleteRefInDelta(j, sym, sti)
         if sti in self.Final:
             self.Final.remove(sti)
@@ -330,7 +343,7 @@ class SFT(GFT):
                 toDel.add(s)
                 toAdd.add(s-1)
         self.Final = self.Final - toDel | toAdd
-        for j in xrange(sti + 1, len(self.States)):
+        for j in range(sti + 1, len(self.States)):
             if j in self.delta:
                 self.delta[j - 1] = self.delta[j]
                 del self.delta[j]
@@ -350,7 +363,7 @@ class SFT(GFT):
         new.setSigma(self.Sigma)
         new.setOutput(self.Output)
         new.States = copy.deepcopy(self.States)
-        for s in self.delta.keys():
+        for s in list(self.delta.keys()):
             if s in self.Initial:
                 new.addInitial(s)
             if self.finalP(s):
@@ -451,16 +464,6 @@ class SFT(GFT):
         :param list sts: list of states"""
         self.Initial = set(sts)
 
-    def addOutput(self, sym):
-        """Add a new symbol to the output alphabet
-
-        There is no problem with duplicate symbols because Output is a Set. No symbol Epsilon can be added
-
-        :param str sym: symbol or regular expression to be added"""
-        if sym == Epsilon:
-            raise common.DFAepsilonRedefinition()
-        self.Output.add(sym)
-
     def addTransition(self, stsrc, symi, symo, sti2):
         """Adds a new transition
 
@@ -588,7 +591,7 @@ class SFT(GFT):
         aut.States = copy.copy(self.States)
         aut.setInitial(self.Initial)
         aut.setFinal(self.Final)
-        for s in self.delta.keys():
+        for s in list(self.delta.keys()):
             aut.delta[s] = {}
             for c in self.delta[s]:
                 aut.delta[s][c] = set([x for (_, x) in self.delta[s][c]])
@@ -867,7 +870,7 @@ class SFT(GFT):
 
         :param DFA|NFA other: the automaton used as a filter of the output
         :rtype: SFT"""
-        return (self.inverse() & other).inverse()
+        return (self.inverse().inIntersection(other)).inverse()
 
     def outIntersection(self, other):
         """Conjunction of transducer and automaton: X & Y using output intersect operation.
@@ -970,7 +973,7 @@ class SFT(GFT):
         Note: original
         :param tuple wp: pair of words
         :rtype: bool"""
-        import fl
+        from . import fl
         (win, wout) = wp
         inT = self.inIntersectionSlow(fl.FL([win]).trieFA().toNFA())
         return not inT.outIntersection(fl.FL([wout]).trieFA().toNFA()).emptyP()
@@ -980,7 +983,7 @@ class SFT(GFT):
 
         :param tuple wp: pair of words
         :rtype: bool"""
-        import fl
+        from . import fl
         (win, wout) = wp
         inT = self.inIntersection(fl.FL([win]).MADFA())
         return not inT.outIntersection(fl.FL([wout]).MADFA()).emptyP()
@@ -989,6 +992,7 @@ class SFT(GFT):
         """Tests if the relation realized  the empty transducer
 
         :rtype: bool"""
+        x = self.toInNFA()
         return self.toInNFA().emptyP()
 
     def nonEmptyW(self):
@@ -1039,7 +1043,9 @@ class SFT(GFT):
                 else:
                     return ""
 
-        def _newSValue((v1, v2), (r1, r2)):
+        def _newSValue(xxx_todo_changeme, xxx_todo_changeme1):
+            (v1, v2) = xxx_todo_changeme
+            (r1, r2) = xxx_todo_changeme1
             a, b = concatN((v1, v2), (r1, r2))
             s = _suffix(a, b)
             if s == Epsilon:
