@@ -32,9 +32,9 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA."""
 
-from comboperations import *
-from reex import *
-from fa import DFA
+from . comboperations import *
+from . reex import *
+from . fa import DFA
 
 # Useful automata
 
@@ -92,7 +92,7 @@ witnessDFA = {"toDFA": [("toDFAWCMF", "int"),
               "disjWStar": [("disjWStarWC", "int", "int")]}
 
 
-def toDFAWC2MF(m=5):
+def toDFAWC2MF(m=5,Sigma=["a","b"]):
     """ Worst case automata for toDFA(NFA) with n > 2, k=2
     ..seealso::
     A. R. Meyer and M. J. Fischer. Economy of description by automata,
@@ -105,19 +105,34 @@ def toDFAWC2MF(m=5):
     :rtype: DFA"""
 
     if m < 3:
-        raise TestsError("number of states must be greater than 2")
+        raise TstError("Number of states must be greater than 2")
+    if len(Sigma) != 2:
+        raise TstError("Alphabet must be binary")
     f = NFA()
-    f.setSigma(["a", "b"])
-    f.States = range(m)
+    f.setSigma(Sigma)
+    f.States = list(range(m))
     f.setInitial([0])
     f.addFinal(0)
-    f.addTransition(0, "a", 1)
+    f.addTransition(0, Sigma[0], 1)
     for i in range(1, m):
-        f.addTransition(i, "a", (i + 1) % m)
-        f.addTransition(i, "b", i)
-        f.addTransition(i, "b", 0)
+        f.addTransition(i, Sigma[0], (i + 1) % m)
+        f.addTransition(i, Sigma[1], i)
+        f.addTransition(i, Sigma[1], 0)
     return f
 
+def toDFAWCReMF(m=5,Sigma=["a","b"]):
+    """
+    Same as toDFAWC2MF as regular expression
+    :param m:
+    :return:
+    """
+    if len(Sigma) != 2:
+        raise TstError("Alphabet must be binary")
+    sig=[atom(i) for i in Sigma]
+    r0 = disj(sig[1],concat(concat(sig[0],star(sig[1])),disj(sig[0],sig[1])))
+    for j in range(1,m-1):
+        r0 = concat(concat(sig[0],star(sig[1])),disj(r0,sig[1]))
+    return star(r0)
 
 def toDFAWC2(m=5):
     """ Worst case automata for toDFA(NFA) with n > 2, k=2
@@ -131,10 +146,10 @@ def toDFAWC2(m=5):
     :rtype: DFA"""
 
     if m < 3:
-        raise TestsError("number of states must be greater than 2")
+        raise TstError("number of states must be greater than 2")
     f = NFA()
     f.setSigma(["a", "b"])
-    f.States = range(m)
+    f.States = list(range(m))
     f.setInitial([0])
     f.addFinal(m - 1)
     f.addTransition(0, "a", 1)
@@ -158,10 +173,10 @@ def toDFAWC3(m=5):
     :rtype: DFA"""
 
     if m < 3:
-        raise TestsError("number of states must be greater than 2")
+        raise TstError("number of states must be greater than 2")
     f = NFA()
     f.setSigma(["a", "b", "c"])
-    f.States = range(m)
+    f.States = list(range(m))
     f.setInitial([0])
     f.addFinal(0)
     f.addTransition(0, "a", 1)
@@ -191,7 +206,7 @@ def reversalWC3M(m=5):
     :rtype: DFA"""
 
     if m < 3:
-        raise TestsError("number of states  must be greater than 2")
+        raise TstError("number of states  must be greater than 2")
     return toDFAWC3(m).reversal()
 
 
@@ -219,11 +234,11 @@ def starWC(m=5):
     :rtype: DFA"""
 
     if m < 3:
-        raise TestsError("number of states must be greater than 2")
+        raise TstError("number of states must be greater than 2")
         # for m=2, L=\{w\in\{a,b\}*| |w|a odd \}
     f = DFA()
     f.setSigma(["a", "b"])
-    f.States = range(m)
+    f.States = list(range(m))
     f.setInitial(0)
     f.addFinal(m - 1)
     f.addTransition(0, "a", 1)
@@ -246,10 +261,10 @@ def starWCM(m=5):
     :rtype: DFA"""
 
     if m < 3:
-        raise TestsError("number of states must be greater than 2")
+        raise TstError("number of states must be greater than 2")
     f = DFA()
     f.setSigma(["a", "b"])
-    f.States = range(m)
+    f.States = list(range(m))
     f.setInitial(0)
     f.addFinal(m - 1)
     f.addTransition(m - 1, "a", 0)
@@ -289,10 +304,10 @@ def concatWCM(m=4, n=4):
     :rtype: (DFA, DFA)"""
 
     if n < 2 or m < 2:
-        raise TestsError("number of states must be both greater than 1")
+        raise TstError("number of states must be both greater than 1")
     d1, d2 = DFA(), DFA()
     d1.setSigma(["a", "b"])
-    d1.States = range(m)
+    d1.States = list(range(m))
     d1.setInitial(0)
     d1.addFinal(m - 1)
     d1.addTransition(m - 1, "b", 0)
@@ -301,7 +316,7 @@ def concatWCM(m=4, n=4):
         d1.addTransition(i, "a", i)
         d1.addTransition(i, "b", i + 1)
     d2.setSigma(["a", "b"])
-    d2.States = range(n)
+    d2.States = list(range(n))
     d2.setInitial(0)
     d2.addFinal(n - 1)
     d2.addTransition(n - 1, "a", n - 1)
@@ -327,10 +342,10 @@ def concatWC(m=6, n=6):
     :returns: two dfas 
     :rtype: (DFA, DFA)"""
     if n < 2 or m < 2:
-        raise TestsError("number of states must both  greater than 1")
+        raise TstError("number of states must both  greater than 1")
     d1, d2 = DFA(), DFA()
     d1.setSigma(["a", "b", "c"])
-    d1.States = range(m)
+    d1.States = list(range(m))
     d1.setInitial(0)
     d1.addFinal(m - 1)
     for i in range(m):
@@ -338,7 +353,7 @@ def concatWC(m=6, n=6):
         d1.addTransition(i, "b", 0)
         d1.addTransition(i, "c", i)
     d2.setSigma(["a", "b", "c"])
-    d2.States = range(n)
+    d2.States = list(range(n))
     d2.setInitial(0)
     d2.addFinal(n - 1)
     for i in range(n):
@@ -361,10 +376,10 @@ def concatWCB(m=4, n=4):
     :rtype: (DFA, DFA)"""
 
     if n < 2 or m < 2:
-        raise TestsError("number of states must be both greater than 1")
+        raise TstError("number of states must be both greater than 1")
     d1, d2 = DFA(), DFA()
     d1.setSigma(["a", "b"])
-    d1.States = range(m)
+    d1.States = list(range(m))
     d1.setInitial(0)
     d1.addFinal(m - 1)
     d1.addTransition(m - 1, "b", 0)
@@ -373,7 +388,7 @@ def concatWCB(m=4, n=4):
         d1.addTransition(i, "a", i)
         d1.addTransition(i, "b", i + 1)
     d2.setSigma(["a", "b"])
-    d2.States = range(n)
+    d2.States = list(range(n))
     d2.setInitial(0)
     d2.addFinal(n - 1)
     d2.addTransition(n - 1, "a", 0)
@@ -401,17 +416,17 @@ def interWC(m=6, n=5):
     :returns: two dfas 
     :rtype: (DFA, DFA)"""
     if n < 2 or m < 2:
-        raise TestsError("number of states must be both greater than 1")
+        raise TstError("number of states must be both greater than 1")
     d1, d2 = DFA(), DFA()
     d1.setSigma(["a", "b"])
-    d1.States = range(m)
+    d1.States = list(range(m))
     d1.setInitial(0)
     d1.addFinal(0)
     for i in range(m):
         d1.addTransition(i, "a", (i + 1) % m)
         d1.addTransition(i, "b", i)
     d2.setSigma(["a", "b"])
-    d2.States = range(m)
+    d2.States = list(range(m))
     d2.setInitial(0)
     d2.addFinal(0)
     for i in range(n):
@@ -433,10 +448,10 @@ def disjWC(m=6, n=5):
     :rtype: (DFA, DFA)"""
 
     if n < 2 or m < 2:
-        raise TestsError("number of states must be both greater than 1")
+        raise TstError("number of states must be both greater than 1")
     d1, d2 = DFA(), DFA()
     d1.setSigma(["a", "b"])
-    d1.States = range(m)
+    d1.States = list(range(m))
     d1.setInitial(0)
     d1.addTransition(0, "a", 1)
     d1.addTransition(0, "b", 0)
@@ -445,7 +460,7 @@ def disjWC(m=6, n=5):
         d1.addTransition(i, "b", i)
         d1.addFinal(i)
     d2.setSigma(["a", "b"])
-    d2.States = range(m)
+    d2.States = list(range(m))
     d2.setInitial(0)
     d2.addTransition(0, "b", 1)
     d2.addTransition(0, "a", 0)
@@ -467,10 +482,10 @@ def reversalMB(m=8):
     :returns: a dfa
     :rtype: DFA"""
     if m < 3:
-        raise TestsError("number of states must be greater than 2")
+        raise TstError("number of states must be greater than 2")
     d = DFA()
     d.setSigma(["a", "b"])
-    d.States = range(m)
+    d.States = list(range(m))
     d.setInitial(0)
     for i in range(m):
         if i == m - 1:
@@ -497,10 +512,10 @@ def reversalWC3L(m=5):
     :rtype: DFA"""
 
     if m < 3:
-        raise TestsError("number of states must be greater than 2")
+        raise TstError("number of states must be greater than 2")
     f = DFA()
     f.setSigma(["a", "b", "c"])
-    f.States = range(m)
+    f.States = list(range(m))
     f.setInitial(0)
     f.addFinal(0)
     f.addTransition(0, "b", 1)
@@ -524,12 +539,12 @@ def reversalternaryWC(m=5):
         :returns: a dfa
         :rtype: DFA"""
     if m < 3:
-        raise TestsError("number of states must be greater than 2")
+        raise TstError("number of states must be greater than 2")
     d = DFA()
     d.setSigma(["a", "b", "c"])
     d.setInitial(0)
     d.addFinal(0)
-    d.States = range(m)
+    d.States = list(range(m))
     d.addTransition(0, "a", m - 1)
     d.addTransition(0, "c", 0)
     d.addTransition(0, "b", 0)
@@ -553,10 +568,10 @@ def reversalbinaryWC(m=5):
     @rtype: DFA"""
 
     if m < 2:
-        raise TestsError("number of states must be greater than 1")
+        raise TstError("number of states must be greater than 1")
     d = DFA()
     d.setSigma(["a", "b"])
-    d.States = range(m)
+    d.States = list(range(m))
     d.setInitial(0)
     d.addFinal(m - 1)
     d.addTransition(0, "a", 1)
@@ -596,9 +611,9 @@ def shuffleWC(m=3, n=3):
     :returns: two dfas 
     :rtype: (DFA, DFA)"""
     if n < 2 or m < 2:
-        raise TestsError("number of states must be both greater than 1")
+        raise TstError("number of states must be both greater than 1")
     d1, d2 = DFA(), DFA()
-    d1.States = range(m)
+    d1.States = list(range(m))
     d1.setSigma(["a", "b", "c", "d", "f"])
     d1.setInitial(0)
     d1.addFinal(0)
@@ -609,7 +624,7 @@ def shuffleWC(m=3, n=3):
         d1.addTransition(i, "d", i)
         if i != 0:
             d1.addTransition(i, "f", i)
-    d2.States = range(n)
+    d2.States = list(range(n))
     d2.setSigma(["a", "b", "c", "d", "f"])
     d2.setInitial(0)
     d2.addFinal(0)
@@ -636,9 +651,9 @@ def starDisjWC(m=6, n=5):
     :rtype: (DFA,DFA)"""
 
     if n < 2 or m < 2:
-        raise TestsError("number of states must be both greater than 1")
+        raise TstError("number of states must be both greater than 1")
     d1, d2 = DFA(), DFA()
-    d1.States = range(m)
+    d1.States = list(range(m))
     d1.setSigma(["a", "b", "c"])
     d1.setInitial(0)
     d1.addFinal(0)
@@ -648,7 +663,7 @@ def starDisjWC(m=6, n=5):
         if i != 0:
             d1.addTransition(i, "c", i)
     d1.addTransition(0, "c", 1)
-    d2.States = range(n)
+    d2.States = list(range(n))
     d2.setSigma(["a", "b", "c"])
     d2.setInitial(0)
     d2.addFinal(0)
@@ -673,10 +688,10 @@ def starInterBC(m=3, n=3):
     :rtype: (DFA,DFA)"""
 
     if n < 2 or m < 2:
-        raise TestsError("number of states must be both greater than 1")
+        raise TstError("number of states must be both greater than 1")
     d1, d2 = DFA(), DFA()
     d1.setSigma(["a", "b", "c", "d", "e"])
-    d1.States = range(m)
+    d1.States = list(range(m))
     d1.setInitial(0)
     d1.addFinal(m - 1)
     for i in range(m):
@@ -686,7 +701,7 @@ def starInterBC(m=3, n=3):
         d1.addTransition(i, "d", i)
         d1.addTransition(i, "e", i)
     d2.setSigma(["a", "b", "c", "d", "e"])
-    d2.States = range(n)
+    d2.States = list(range(n))
     d2.setInitial(0)
     d2.addFinal(n - 1)
     for i in range(n):
@@ -718,10 +733,10 @@ def disjWStarWC(m=6, n=5):
   :rtype: (DFA,DFA)"""
 
     if n < 3 or m < 3:
-        raise TestsError("number of states must be greater than 2")
+        raise TstError("number of states must be greater than 2")
     f1 = DFA()
     f1.setSigma(["a", "b", "c"])
-    f1.States = range(m)
+    f1.States = list(range(m))
     f1.setInitial(0)
     f1.addFinal(m - 1)
     f1.addTransition(0, "a", 1)
@@ -733,7 +748,7 @@ def disjWStarWC(m=6, n=5):
         f1.addTransition(i, "c", i)
     f2 = DFA()
     f2.setSigma(["a", "b", "c"])
-    f2.States = range(n)
+    f2.States = list(range(n))
     f2.setInitial(0)
     f2.addFinal(n - 1)
     for i in range(n):
@@ -762,17 +777,17 @@ def unionWCTk2(m=6, n=6):
     :rtype: (DFA,DFA)"""
 
     if n < 2 or m < 2:
-        raise TestsError("number of states must both  greater than 1")
+        raise TstError("number of states must both  greater than 1")
     d1, d2 = DFA(), DFA()
     d1.setSigma(["a", "b"])
-    d1.States = range(m)
+    d1.States = list(range(m))
     d1.setInitial(0)
     d1.addFinal(0)
     d1.addTransition(m - 1, "a", 0)
     for i in range(0, m - 1):
         d1.addTransition(i, "b", i + 1)
     d2.setSigma(["a", "b"])
-    d2.States = range(n)
+    d2.States = list(range(n))
     d2.setInitial(0)
     d2.addFinal(n - 1)
     d2.addTransition(n - 1, "b", n - 1)
@@ -792,17 +807,17 @@ def unionWCT2(n=6):
     :rtype: (DFA,DFA)"""
     m = 1
     if n < 2:
-        raise TestsError("number of states must both  greater than 1")
+        raise TstError("number of states must both  greater than 1")
     d1, d2 = DFA(), DFA()
     d1.setSigma(["a", "b", "c"])
-    d1.States = range(m)
+    d1.States = list(range(m))
     d1.setInitial(0)
     d1.addFinal(0)
     d1.addTransition(0, "b", 0)
     d1.addTransition(0, "c", 0)
 
     d2.setSigma(["a", "b", "c"])
-    d2.States = range(n)
+    d2.States = list(range(n))
     d2.setInitial(0)
     d2.addFinal(n - 1)
     d2.addTransition(0, "a", 0)
@@ -825,10 +840,10 @@ def unionWCT(m=6, n=6):
     :rtype: (DFA,DFA)"""
 
     if n < 2 or m < 2:
-        raise TestsError("number of states must both  greater than 1")
+        raise TstError("number of states must both  greater than 1")
     d1, d2 = DFA(), DFA()
     d1.setSigma(["a", "b", "c"])
-    d1.States = range(m)
+    d1.States = list(range(m))
     d1.setInitial(0)
     d1.addFinal(m - 1)
     d1.addTransition(0, "a", 1)
@@ -838,7 +853,7 @@ def unionWCT(m=6, n=6):
         d1.addTransition(i, "b", 0)
         d1.addTransition(i, "c", i)
     d2.setSigma(["a", "b", "c"])
-    d2.States = range(n)
+    d2.States = list(range(n))
     d2.setInitial(0)
     d2.addFinal(n - 1)
     d2.addTransition(0, "a", 0)
@@ -861,17 +876,17 @@ def concatWCT2(n=6):
     :rtype: (DFA,DFA)"""
     m = 1
     if n < 2:
-        raise TestsError("number of states must both  greater than 1")
+        raise TstError("number of states must both  greater than 1")
     d1, d2 = DFA(), DFA()
     d1.setSigma(["a", "b", "c"])
-    d1.States = range(m)
+    d1.States = list(range(m))
     d1.setInitial(0)
     d1.addFinal(0)
     d1.addTransition(0, "b", 0)
     d1.addTransition(0, "c", 0)
 
     d2.setSigma(["a", "b", "c"])
-    d2.States = range(n)
+    d2.States = list(range(n))
     d2.setInitial(0)
     d2.addFinal(n - 1)
     d2.addTransition(0, "a", 0)
@@ -892,10 +907,10 @@ def concatWCT3(m=6):
     :rtype: (DFA,DFA)"""
     n = 1
     if m < 2:
-        raise TestsError("number of states must both  greater than 1")
+        raise TstError("number of states must both  greater than 1")
     d1, d2 = DFA(), DFA()
     d1.setSigma(["a", "b", "c"])
-    d1.States = range(m)
+    d1.States = list(range(m))
     d1.setInitial(0)
     d1.addFinal(m - 1)
     d1.addTransition(0, "a", 0)
@@ -908,7 +923,7 @@ def concatWCT3(m=6):
         d1.addTransition(i, "c", (i + 1) % m)
         d1.addTransition(i, "a", i)
     d2.setSigma(["a", "b", "c"])
-    d2.States = range(n)
+    d2.States = list(range(n))
     d2.setInitial(0)
     d2.addFinal(0)
     d2.addTransition(0, "c", 0)
@@ -928,10 +943,10 @@ def concatWCT(m=6, n=6):
     :rtype: (DFA,DFA)"""
 
     if n < 2 or m < 2:
-        raise TestsError("number of states must both  greater than 1")
+        raise TstError("number of states must both  greater than 1")
     d1, d2 = DFA(), DFA()
     d1.setSigma(["a", "b", "c"])
-    d1.States = range(m)
+    d1.States = list(range(m))
     d1.setInitial(0)
     d1.addFinal(m - 1)
     d1.addTransition(0, "a", 1)
@@ -941,7 +956,7 @@ def concatWCT(m=6, n=6):
         d1.addTransition(i, "b", 0)
         d1.addTransition(i, "c", i)
     d2.setSigma(["a", "b", "c"])
-    d2.States = range(n)
+    d2.States = list(range(n))
     d2.setInitial(0)
     d2.addFinal(n - 1)
     d2.addTransition(0, "a", 0)
@@ -963,10 +978,10 @@ def starWCT(m=5):
     :returns: dfa 
     :rtype: DFA"""
     if m < 3:
-        raise TestsError("number of states must be greater than 2")
+        raise TstError("number of states must be greater than 2")
     f = DFA()
     f.setSigma(["a", "b"])
-    f.States = range(m)
+    f.States = list(range(m))
     f.setInitial(0)
     f.addFinal(m - 1)
     f.addTransition(0, "a", 1)
@@ -984,10 +999,10 @@ def starWCT1(m=5):
     :returns: dfa 
     :rtype: DFA"""
     if m < 3:
-        raise TestsError("number of states must be greater than 2")
+        raise TstError("number of states must be greater than 2")
     f = DFA()
     f.setSigma(["a", "b"])
-    f.States = range(m)
+    f.States = list(range(m))
     f.setInitial(0)
     f.addFinal(m - 1)
     f.addTransition(0, "b", 0)
@@ -1010,9 +1025,9 @@ def universal(n, l= None, Finals=None, dialect=False, d=None):
     :rtype: DFA
     """
     if n < 3:
-        raise TestsError("number of states must be greater than 2")
+        raise TstError("number of states must be greater than 2")
     u = DFA()
-    u.States = range(n)
+    u.States = list(range(n))
     if l is None:
         l = ("a", "b", "c")
     u.setSigma(list(l))
@@ -1057,17 +1072,17 @@ def dcMilano1(n):
     :rtype: NFA"""
     new = fa.NFA()
     st = []
-    for _ in xrange(3 * n):
+    for _ in range(3 * n):
         s = new.addState()
         st.append(s)
     new.setInitial([st[n - 1]])
-    for s in xrange(n):
+    for s in range(n):
         new.addTransition(st[n - 1], 'c', st[s])
-    for s in xrange(3):
-        for r in xrange(n - 1):
+    for s in range(3):
+        for r in range(n - 1):
             new.addTransition(st[(n * s) + r], 'a', st[(n * s) + r + 1])
         new.addTransition(st[(n * s) + n - 1], 'a', st[0])
-    for s in xrange(n):
+    for s in range(n):
         new.addTransition(st[s], 'b', st[s + n])
         new.addTransition(st[s + n], 'b', st[s + 2 * n])
         new.addTransition(st[s + 2 * n], 'b', st[s])
@@ -1085,16 +1100,16 @@ def dcMilano2(n):
     :rtype: NFA"""
     new = fa.NFA()
     st = []
-    for _ in xrange(3 * n):
+    for _ in range(3 * n):
         s = new.addState()
         st.append(s)
-    for s in xrange(n):
+    for s in range(n):
         new.addInitial(st[s])
-    for s in xrange(3):
-        for r in xrange(n - 1):
+    for s in range(3):
+        for r in range(n - 1):
             new.addTransition(st[(n * s) + r], 'a', st[(n * s) + r + 1])
         new.addTransition(st[(n * s) + n - 1], 'a', st[0])
-    for s in xrange(n):
+    for s in range(n):
         new.addTransition(st[s], 'b', st[s + n])
         new.addTransition(st[s + n], 'b', st[s + 2 * n])
         new.addTransition(st[s + 2 * n], 'b', st[s])
@@ -1115,10 +1130,10 @@ def suffWCe(m=3):
 
      """
     if m< 3:
-        raise TestsError("number of states must be greater than 2")
+        raise TstError("number of states must be greater than 2")
     f = DFA()
     f.setSigma(["a", "b"])
-    f.States = range(m)
+    f.States = list(range(m))
     f.setInitial(0)
     f.addFinal(0)
     f.addTransition(0, "a", 1)
@@ -1139,10 +1154,10 @@ def suffWCd(m=3):
     ..seealso: as above
     """
     if m< 3:
-        raise TestsError("number of states must be greater than 2")
+        raise TstError("number of states must be greater than 2")
     f = DFA()
     f.setSigma(["a", "b"])
-    f.States = range(m)
+    f.States = list(range(m))
     f.setInitial(0)
     f.addFinal(0)
     f.addTransition(0, "a", 1)
@@ -1161,10 +1176,10 @@ def suffWCsynt(m=3):
 
   """
   if m< 3:
-     raise TestsError("number of states must be greater than 2")
+     raise TstError("number of states must be greater than 2")
   f = DFA()
   f.setSigma(["a", "b", "c", "d", "e"])
-  f.States = range(m)
+  f.States = list(range(m))
   f.setInitial(0)
   f.addFinal(m-1)
   f.addTransition(0, "a", 0)
@@ -1204,10 +1219,10 @@ def booleanWCSymGrp(m=3):
    	ICALP (2) 2014: 1-12
   """
   if m< 3:
-     raise TestsError("number of states must be greater than 2")
+     raise TstError("number of states must be greater than 2")
   f = DFA()
   f.setSigma(["a", "b"])
-  f.States = range(m)
+  f.States = list(range(m))
   f.setInitial(0)
   f.addFinal(0)
   f.addFinal(1)
@@ -1225,10 +1240,10 @@ def suffFreeSyntWC(m=5):
 
     """
     if m < 5:
-     raise TestsError("number of states must be greater than 2")
+     raise TstError("number of states must be greater than 2")
     f = DFA()
     f.setSigma(["a", "b", "c","d", "e"])
-    f.States = range(m)
+    f.States = list(range(m))
     f.setInitial(0)
     f.addFinal(1)
     f.addTransition(0,"a",m-1)
@@ -1270,7 +1285,7 @@ def shuffle_ReBC(n=3):
     """
 
     if n < 2:
-        raise TestsError("at least n=2")
+        raise TstError("at least n=2")
     rs = star(concat(atom('a0'), atom('b0')))
     for i in range(1, n):
         rs = shuffle(rs,star(concat(atom('a'+str(i)),atom('b'+str(i)))))
@@ -1279,10 +1294,10 @@ def shuffle_ReBC(n=3):
 def shuffle_ReBCF(n=3):
     """
     :param n:
-    :return:
+    :return:regexp
     """
     if n<2:
-        raise TestsError("at least n=2")
+        raise TstError("at least n=2")
     rs = concat(atom('a0'),atom('b0'))
     for i in range(1,n):
         rs = shuffle(rs,concat(atom('a'+str(i)),atom('b'+str(i))))
@@ -1295,7 +1310,7 @@ def shuffle_NFABC(n=3):
     :rtype: atom
     """
     if n<2:
-        raise TestsError("at least n=2")
+        raise TstError("at least n=2")
     ns = star(concat(atom('a0'), atom('b0'))).toNFA()
     for i in range(1, n):
         ni = star(concat(atom('a'+str(i)), atom('b'+str(i)))).toNFA()
@@ -1305,12 +1320,12 @@ def shuffle_NFABC(n=3):
 
 def shuffle_ReWC(n=4):
     """
-
+    Regular expressions for worst case partial derivative automaton from shuffle operator
     :param int n:
     :rtype: atom
     """
     if n<2:
-        raise TestsError("at least n=2")
+        raise TstError("at least n=2")
     rs = atom('a0')
     for i in range(1, n):
         rs = shuffle(rs, atom('a'+str(i)))
@@ -1318,6 +1333,12 @@ def shuffle_ReWC(n=4):
 
 
 def intersectionReWC(n, sym = 'a'):
+    """
+    Regular expressions for "bad" case partial derivative automaton from intersection  operator
+    :param n: number of letters
+    :param sym: alphabetic symbol
+    :return: regexp
+    """
     if n == 0:
         return epsilon()
     if n == 1:
@@ -1333,8 +1354,34 @@ def intersectionReWC(n, sym = 'a'):
         r2 = sstar(sdisj(frozenset(s2)))
         return sconj(frozenset([r1,r2]))
 
+def intersectionRetoReLB(m=3,n=3, Sigma=["a","b"]):
+    """
+    Regular expressions for a lower bound for intersection.
+    ...seealso:
+        H. Gruber and M. Holzer, Tight Bounds on the Descriptional Complexity of Regular Expressions, DLT 2009
+    :param m: alphabetic size
+    :param n: alphabetic size
+    :return: regular expression for the intersection
+    """
+    a=atom(Sigma[0])
+    b=atom(Sigma[1])
+    bs=star(b)
+    r1= a
+    for i in range(m):
+        r1 = concat(r1,concat(bs,a))
+    return star(disj(b,r1))
 
+
+
+#(b + ((((a b*) a) b*) a))* & (a + ((((b a*) b) a*) b))*
+
+# reveversible DFA
 def revFibonnacci(n):
+    """
+    Worst-case for reversible DFA
+    :param n: number of states
+    :return: revdfa
+    """
     a = DFA ()
     a.setSigma (["a", "b"]),
     for i in range (n):
